# -*- coding: utf-8 -*-
#
#  ConfigViewer
#
#  $Id: plugin.py,v 1.3 2012/01/08 15:30:00 dre Exp $
#
#  Coded by dre (c) 2011
#  Coding idea and design by dre
#  Support: www.dreambox-tools.info
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from Components.config import config, ConfigSubsection, ConfigSelection, ConfigYesNo, configfile, getConfigListEntry
from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
from Components.Pixmap import Pixmap
from Plugins.Plugin import PluginDescriptor
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.Standby import TryQuitMainloop
from Tools.Directories import fileExists, resolveFilename, SCOPE_CURRENT_SKIN
from Tools.LoadPixmap import LoadPixmap
from enigma import RT_HALIGN_LEFT, RT_HALIGN_RIGHT, eListboxPythonMultiContent, gFont, eConsoleAppContainer
import os, re

# for localized messages
from . import _

config.plugins.configmanager = ConfigSubsection()
config.plugins.configmanager.backuppath = ConfigSelection([("/media/hdd", "/media/hdd"), ("/media/usb", "/media/usb"), ("/media/cf", "/media/cf")], default = "/media/hdd")
config.plugins.configmanager.rememberselection = ConfigYesNo(default=False)

def main(session,**kwargs):
	#check for first run
	if not fileExists("/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/.firstrundone"):
		os.system("touch /usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/.firstrundone")
		os.system("update-rc.d configmanager start 10 3 .")
	session.open(ConfigViewerScreen)

class ConfigMenuList(MenuList):
	def __init__(self, list):
		MenuList.__init__(self, list, False, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 18))
		self.l.setItemHeight(25)
		
def ConfigMenuListEntry(entry, value, selected = False):
	res = [ (entry, value, selected) ]
	if selected is False:
		icon = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "skin_default/icons/lock_off.png"))
	else:
		icon = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "skin_default/icons/lock_on.png"))
	res.append(MultiContentEntryPixmapAlphaTest(pos=(0,0), size=(25,25), png = icon))
	res.append(MultiContentEntryText(pos=(25,0),size=(695,25), font=0, flags=RT_HALIGN_LEFT, text=entry))
	res.append(MultiContentEntryText(text=value))
	return res
	
class CMPluginConfigScreen(Screen, ConfigListScreen, HelpableScreen):
	skin = """
		<screen position="center,center" size="500,110" title="Plugin Config" >
			<widget name="config" position="0,0" size="500,80" scrollbarMode="showOnDemand" />
			<ePixmap alphatest="on" pixmap="skin_default/buttons/button_green.png" position="0,80" size="35,25"/>
			<widget name="SaveText" foregroundColor="#f0f0f0" font="Regular;18" position="40,80" size="500,25" transparent="1"/>
		</screen>"""
		
	def __init__(self, session, args = 0):
		self.skin = CMPluginConfigScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.list = []
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.close, _("Close")),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveConfig, _("Save config")),
		}, -1)
		
		self["SaveText"] = Label(_("Save config and close"))
		
		ConfigListScreen.__init__(self, self.list)
		
		self.list.append(getConfigListEntry(_("Back up path:"), config.plugins.configmanager.backuppath))
		self.list.append(getConfigListEntry(_("Save last selection to config:"), config.plugins.configmanager.rememberselection))
		self["config"].setList(self.list)
		
	def saveConfig(self):
		for x in self.list:
			x[1].save()
		self.close()

class CleanUpWizardScreen(Screen, HelpableScreen):
	skin = """
		<screen position="center,center" size="720,400" title="Clean Up Wizard" >
			<widget name="FoundConfigDescr" position="0,0" size="500,25" halign="left" font="Regular;18" transparent="1" />
			<widget name="FoundConfig" position="500,0" size="100,25" halign="left" font="Regular;18" transparent="1" />
			<widget name="LeftBehindDescr" position="0,25" size="500,25" halign="left" font="Regular;18" transparent="1" />
			<widget name="LeftBehind" position="500,25" size="100,25" halign="left" font="Regular;18" transparent="1" />
			<widget name="menulist" position="0,50" size="720,325" scrollbarMode="showOnDemand" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="0,375" zPosition="10" size="35,25" transparent="1" alphatest="on" />
			<widget name="ButtonRedText" position="35,375" size="300,25" zPosition="10" font="Regular;18" foregroundColor="#f0f0f0" transparent="1" />
		</screen>"""
		
	def __init__(self, session, pluginConfigEntries):
		self.skin = CleanUpWizardScreen.skin
		self.session = session
		self.pluginConfigEntries = pluginConfigEntries
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.selectForRemoval, _("Select entry for removal")),
			"cancel":	(self.close, _("Close screen")),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.removeSelectedEntries, _("Remove selected entries")),
		}, -1)
		
		self.menulist = []
		self.pluginConfigList = []
		self["menulist"] = ConfigMenuList([])
		
		self.selectedEntries = []
		
		self["FoundConfig"] = Label()
		self["FoundConfigDescr"] = Label(_("Config entries found:"))
		
		self["LeftBehind"] = Label()
		self["LeftBehindDescr"] = Label(_("Config entries left behind:"))
		
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Remove selected entries"))
		
		self.foundConfig = "0"
		self.leftBehind = "0"
		
		self["FoundConfig"].setText(self.foundConfig)
		self["LeftBehind"].setText(self.leftBehind)
		
		self.dirlist = ["/usr/lib/enigma2/python/Plugins/Extensions", "/usr/lib/enigma2/python/Plugins/SystemPlugins"]
		
		self.subdirlist = []
		self.subdirlist = self.getSubDirectories(self.dirlist)
		
		for i in range(len(self.subdirlist)):
			for p in range(len(self.subdirlist[i])):
				path = self.dirlist[i] + "/" + self.subdirlist[i][p]
				pyfiles = self.getpyFiles(path)
				for j in range(len(pyfiles)):
					self.content = ""
					checkFile = open(path + "/" + pyfiles[j], "r")
					self.content = checkFile.read()
					checkFile.close()
					configentries = re .findall("config\.(.*?)[\b|\=]", self.content)
					if len(configentries) > 0:
						for k in range(len(configentries)):
							if configentries[k].find(".value") == -1 and configentries[k].find(".addNotifier") == -1 and configentries[k].find("getValue") == -1 and configentries[k].find(".save()") == -1 and configentries[k].find(".index >") == -1:
								self.pluginConfigList.append("config." + configentries[k].strip())

		self.foundConfig = len(self.pluginConfigList)
		self["FoundConfig"].setText(str(self.foundConfig))
			
		for c in self.pluginConfigEntries:
			if c not in self.pluginConfigList:
				sublistFound = False
				sublistcheck = ""
				sublistcheck = re.findall("config\.plugins\.(.*?)\.([0-9])\.(.*?)$", c)
				#a sublistcheck length greater than 0 means we have a ConfigSubList
				if len(sublistcheck) > 0:
					for e in range(len(self.pluginConfigList)):
						sublistFound = False
						pos = self.pluginConfigList[e].find("[")
						if pos != -1:
							closepos = self.pluginConfigList[e].find("]")
							modEntry = "%s.%d%s" %(self.pluginConfigList[e][:pos], int(sublistcheck[0][1]), self.pluginConfigList[e][closepos+1:])
							if c == modEntry:
								sublistFound = True
								break
				if sublistFound == False:
					self.menulist.append(ConfigMenuListEntry(c, None, selected = False))
		self.differenceList = []
		self.differenceList = self.menulist
		self["menulist"].l.setList(self.menulist)
		
		self.leftBehind = len(self.menulist)
		self["LeftBehind"].setText(str(self.leftBehind))
		if self.leftBehind == 0:
			self["ButtonRed"].hide()
			self["ButtonRedText"].hide()
		
	def getSubDirectories(self, dir):
		return [ [subdirectory for subdirectory in os.listdir(directory) if os.path.isdir(os.path.join(directory,subdirectory))] for directory in dir ]

	def getpyFiles(self, path):
		return [ pyfile for pyfile in os.listdir(path)
					if os.path.isfile(os.path.join(path, pyfile)) and pyfile.endswith(".py")]
					
	def removeSelectedEntries(self):
		if fileExists("/etc/enigma2/settings"):
			try:
				origFile = open('/etc/enigma2/settings', 'r')
				self.settingsList = []
				self.settingsList = origFile.readlines()
				
				for i in range(len(self.selectedEntries)):
					for k in range(len(self.settingsList)):
						entrysplit = self.settingsList[k].split('=')
						if entrysplit[0] == self.selectedEntries[i]:
							#a dirty hack
							self.settingsList[k] = "#"
							break
				
				origFile.close()
				
				os.system("touch /etc/enigma2/settings.new")
				os.system("chmod 644 /etc/enigma2/setttings.new")
				configFile = open('/etc/enigma2/settings.new', 'w')	
				for j in range(len(self.settingsList)):
					if self.settingsList[j] != "#":
						configFile.write(self.settingsList[j])
				configFile.close()

				self.doRestart()			
				
			except IOError:
				print "couldn't read settings"
				
	def selectForRemoval(self):
		if len(self.menulist) > 0:
			selectedMenuEntry = self["menulist"].getSelectedIndex()
			if self.menulist[selectedMenuEntry][0][2] == False:
				self.selectedEntries.append(self.menulist[selectedMenuEntry][0][0])
			else:
				for k in range(len(self.selectedEntries)):
					if self.selectedEntries[k] == self.menulist[selectedMenuEntry][0][0]:
						del self.selectedEntries[k]
						break
			self.updateList(jumpToFirst=False)
		
	def updateList(self, jumpToFirst=True):
		self.menulist = []
		self.alreadyAdded = False
	
		for i in range(len(self.differenceList)):
			self.markSelectedEntries(i)

		self["menulist"].l.setList(self.menulist)
		if jumpToFirst:
			self["menulist"].moveToIndex(0)
			
	def markSelectedEntries(self, index):
		self.index = index
		self.alreadyAdded = False
		for j in range(len(self.selectedEntries)):
			if self.differenceList[self.index][0][0] == self.selectedEntries[j]:
				self.menulist.append(ConfigMenuListEntry(self.differenceList[self.index][0][0], None, selected = True))
				self.alreadyAdded = True
				break
		if self.alreadyAdded == False:
			self.menulist.append(ConfigMenuListEntry(self.differenceList[self.index][0][0], None))
		else:
			self.alreadyAdded = False
			
	def doRestart(self):
		restartbox = self.session.openWithCallback(self.restartBox,MessageBox,_("A reboot is required to load settings\nDo you want to reboot the Dreambox now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Reboot Dreambox now?"))
		
	def restartBox(self,answer):
		if answer is True:
			self.session.open(TryQuitMainloop, 2)			
	
class RestoreSelectionScreen(Screen):
	skin = """
		<screen position="center,center" size="300,50" title="Config Manager" >
			<widget name="restoreselectionlist" position="0,0" size="300,50" scrollbarMode="showOnDemand" />
		</screen>"""	
	
	def __init__(self, session):
		self.skin = RestoreSelectionScreen.skin
		self.session = session
		Screen.__init__(self, session)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.runCommand, _("Run selected task")),
			"cancel":	(self.close, _("Close screen")),
		}, -1)
		
		self.entries = []
		self.backupdir = ""
		self.backupdir = config.plugins.configmanager.backuppath.value
		if fileExists("%s/settings.tar.gz" %(self.backupdir)):
			self.entries.append( (_("Restore settings"), 0))
			self.entries.append( (_("Merge back up into settings"), 1))

		self["restoreselectionlist"] = MenuList(self.entries)
		
	def runCommand(self):
		menuitem = self["restoreselectionlist"].getCurrent()[1]
		#restore settings
		if menuitem == 0:
			self.result = os.system("tar -xzf %s/settings.tar.gz -C /etc/enigma2/" %(self.backupdir))		
			if (int(self.result)/256)==0:
				self.doRestart()
		#merge back up	
		elif menuitem == 1:
			self.result = os.system("tar -xzf %s/settings.tar.gz -C /tmp/" %(self.backupdir))
			if (int(self.result)/256)==0:
				self.doMerge()
		
	def doMerge(self):
		self.writeFile = False
		if fileExists("/etc/enigma2/settings") and fileExists("/tmp/settings.new"):
			try:
				buf = []
				backUpFile = open('/tmp/settings.new', 'r')
				mergeList = backUpFile.readlines()
				configFile = open('/etc/enigma2/settings', 'r')
				self.writeFile = True

				for line in configFile:
					for l in range(len(mergeList)):
						if mergeList[l] != '#' and mergeList[l] != '\n':
							backupsplit = mergeList[l].split('=')
							if line.startswith(backupsplit[0]):
								line = line.replace(line, mergeList[l])
								#kinda remove the line to be able to add all lines that are not in configFile
								mergeList[l] = "#"
								break
					buf.append(line)
					
				#this is needed to add lines that are not present in configFile
				for k in range(len(mergeList)):
					if mergeList[k] != '#' and mergeList[k] != '\n':
						buf.append(mergeList[k])
								
			except IOError:
				print "couldn't read settings"
			backUpFile.close()
			configFile.close()
			
			if self.writeFile:
				os.system("touch /etc/enigma2/settings.new")
				os.system("chmod 644 /etc/enigma2/setttings.new")
				configFile = open('/etc/enigma2/settings.new', 'w')	
				for line in buf:
					configFile.write(line)
				configFile.close()
		
				self.doRestart()
	
	def doRestart(self):
		restartbox = self.session.openWithCallback(self.restartBox,MessageBox,_("A reboot is required to load settings\nDo you want to reboot the Dreambox now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Reboot Dreambox now?"))
		
	def restartBox(self,answer):
		if answer is True:
			self.session.open(TryQuitMainloop, 2)				

class ConfigBackUpScreen(Screen,HelpableScreen):
	skin = """
		<screen position="center,center" size="620,220" title="Config Manager - Back Up">
			<widget name="progresstext" position="0,0" size="620,22" halign="left" font="Regular;18" transparent="1" />
			<widget name="backedUp" position="0,25" size="620,175" scrollbarMode="showOnDemand" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="0,200" zPosition="10" size="35,25" transparent="1" alphatest="on" />
			<widget name="ButtonRedText" position="35,200" size="300,25" zPosition="10" font="Regular;18" foregroundColor="#f0f0f0" transparent="1" />
		</screen>"""
	
	def __init__(self, session, selection):
		self.skin = ConfigBackUpScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.closeAndClean, _("Close screen")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.close, _("Close screen")),
		}, -1)
		
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Close"))	
		
		self.selection = selection
		self.backedUpList = []
		self["backedUp"] = MenuList(self.backedUpList)
		
		total = len(self.selection)
		
		self["progresstext"] = Label(_("0/%s backed up") %(total))
		
		counter = 0
		if fileExists("/etc/enigma2/settings"):
			try:
				configFile = open('/etc/enigma2/settings', 'r')
				for i in range(len(self.selection)):
					for line in configFile:
						if line.startswith(self.selection[i]):
							counter = counter+1
							self["progresstext"].setText(_("%s/%s backed up") %(counter, total))
							self.backedUpList.append(line)
							self["backedUp"].l.setList(self.backedUpList)
							break
				os.system("touch /tmp/settings.new")
				os.system("chmod 644 /tmp/setttings.new")
				
				backUpFile = open('/tmp/settings.new', 'w')	
				for k in range(len(self.backedUpList)):
					backUpFile.write(self.backedUpList[k])
				backUpFile.close()
				
				self.backupdir = ""
				self.backupdir = config.plugins.configmanager.backuppath.value
				os.system("tar -zcf %s/settings.tar.gz -C /tmp settings.new" %(self.backupdir))
				
			except IOError:
				print "couldn't read settings"
			configFile.close()
			
	def closeAndClean(self):
		os.system("rm -f /tmp/settings.new")
		self.close()
		
class ConfigViewerScreen(Screen,HelpableScreen):
	skin = """
		<screen position="center,center" size="720,450" title="Config Manager" >
			<widget name="menulist" position="0,0" size="720,225" scrollbarMode="showOnDemand" />
			<ePixmap alphatest="on" pixmap="skin_default/progress_small.png" position="80,230" size="560,3"/>
			<widget name="entryinfo" position="0,235" size="720,100" halign="left" font="Regular;18" transparent="1" />
			<ePixmap alphatest="on" pixmap="skin_default/progress_small.png" position="80,340" size="560,3"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_0.png" position="10,350" size="35,25"/>
			<widget name="allEntries" foregroundColor="#f0f0f0" font="Regular;18" position="50,350" size="300,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_1.png" position="10,375" size="35,25"/>
			<widget name="plugEntries" foregroundColor="#f0f0f0" font="Regular;18" position="50,375" size="300,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_2.png" position="370,350" size="35,25"/>
			<widget name="enigmaEntries" foregroundColor="#f0f0f0" font="Regular;18" position="410,350" size="300,25" transparent="1"/>
			<ePixmap alphatest="on" pixmap="skin_default/buttons/key_3.png" position="370,375" size="35,25"/>
			<widget name="merlinEntries" foregroundColor="#f0f0f0" font="Regular;18" position="410,375" size="300,25" transparent="1"/>
			<widget name="ButtonRedText" position="50,400" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="19,400" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonGreenText" position="50,425" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="19,425" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonYellowText" position="410,400" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonYellow" pixmap="skin_default/buttons/button_yellow.png" position="379,400" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonBlueText" position="410,425" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonBlue" pixmap="skin_default/buttons/button_blue.png" position="379,425" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""
		
	def __init__(self, session, args = 0):
		self.skin = ConfigViewerScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)

		self["NumberActions"] = HelpableActionMap(self, "NumberActions",
		{
			"0":		(self.updateList, _("Show complete config")),
			"1":		(self.showPluginConfigOnly, _("Show plugin config only")),
			"2":		(self.showEnigmaConfigOnly, _("Show Enigma2 config only")),
			"3":		(self.showMerlinConfigOnly, _("Show Merlin3 config only")),
			"7":		(self.selectAll, _("Select all entries")),
			"9":		(self.deselectAll, _("Deselect all entries")),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.removeEntries, _("Delete selected entries")),
			"green":	(self.startBackUp, _("Back up selected entries")),
			"yellow":	(self.showRestoreSelection, _("Restore from back up")),
			"blue":		(self.runCleanUpWizard, _("Run clean up wizard")),
		}, -1)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.selectForBackUp, _("Select entry for back up")),
			"cancel":	(self.closePlugin, _("Close plugin")),
		}, -1)
		
		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
		{
			"menu":		(self.openPluginConfig, _("Open plugin config screen")),
		}, -1)

		self["allEntries"] = Label(_("All entries"))
		self["plugEntries"] = Label(_("Plugin config only"))
		self["enigmaEntries"] = Label(_("Enigma2 config only"))
		self["merlinEntries"] = Label(_("Merlin3 config only"))
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Delete selected entries"))		
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreenText"] = Label(_("Back up selected entries"))
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowText"] = Label(_("Restore from back up"))
		self["ButtonBlue"] = Pixmap()
		self["ButtonBlueText"] = Label(_("Run clean up wizard"))
		
		self["ButtonYellow"].hide()
		self["ButtonYellowText"].hide()
		
		self.setYellowButton()
		
		self.menulist = []
		self.pluginConfigEntries = []
		self["menulist"] = ConfigMenuList([])
		
		self.selectedEntries = []
		if config.plugins.configmanager.rememberselection.value:
			if fileExists("/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection"):
				try:
					selectionFile = open('/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection', 'r')
					selbuf = selectionFile.readline().strip("\n[]").split(",")
					selectionFile.close()
					for k in range(len(selbuf)):
						self.selectedEntries.append(selbuf[k].strip())
				
				except IOError:
					print "couldn't read settings"
		
		self["entryinfo"] = Label()
		
		self.currentView = "ALL"
		
		self.createList()
		
		self["menulist"].onSelectionChanged.append(self.updateEntryinfo)
	
	def openPluginConfig(self):
		self.session.open(CMPluginConfigScreen)
	
	def updateEntryinfo(self):
		selectedMenuEntry = self["menulist"].getSelectedIndex()
		self["entryinfo"].setText(self.menulist[selectedMenuEntry][0][1])
	
	def readConfig(self):
		if fileExists("/etc/enigma2/settings"):
			try:
				configFile = open('/etc/enigma2/settings', 'r')
				lines = []
				for line in configFile:
					if line[0] != '#' and line != '\n':
						val = line.strip().split('=')
						lines.append([val[0],val[1]])
				return lines
			
			except IOError:
				print "couldn't read settings"
				return None

			configFile.close()

	def createList(self):
		self.data = self.readConfig()
		
		for i in range(len(self.data)):
			if self.data[i][0].startswith("config.plugins"):
				self.data[i].append("PLUGIN")
				self.pluginConfigEntries.append(self.data[i][0])
			elif self.data[i][0].startswith("config.merlin"):
				self.data[i].append("MERLIN")
			else:
				self.data[i].append("ENIGMA")

			self.markSelectedEntries(i)
		
		self["menulist"].l.setList(self.menulist)

		self.updateEntryinfo()

	def markSelectedEntries(self, index):
		self.index = index
		self.alreadyAdded = False
		for j in range(len(self.selectedEntries)):
			if self.data[self.index][0] == self.selectedEntries[j]:
				self.menulist.append(ConfigMenuListEntry(self.data[self.index][0], self.data[self.index][1], selected = True))
				self.alreadyAdded = True
				break
		if self.alreadyAdded == False:
			self.menulist.append(ConfigMenuListEntry(self.data[self.index][0], self.data[self.index][1]))
		else:
			self.alreadyAdded = False
			
	def updateList(self, configtype="ALL", jumpToFirst=True):
		self.menulist = []
		self.currentView = configtype
		self.alreadyAdded = False
	
		if self.currentView == "ALL":
			for i in range(len(self.data)):
				self.markSelectedEntries(i)
		else:
			for i in range(len(self.data)):
				if self.data[i][2] == self.currentView:
					self.markSelectedEntries(i)
		
		self["menulist"].l.setList(self.menulist)
		if jumpToFirst:
			self["menulist"].moveToIndex(0)
		self.updateEntryinfo()
	
	def showPluginConfigOnly(self):
		self.updateList("PLUGIN")
	
	def showMerlinConfigOnly(self):
		self.updateList("MERLIN")
		
	def showEnigmaConfigOnly(self):
		self.updateList("ENIGMA")
		
	def selectAll(self):
		for k in range(len(self.menulist)):
			if self.menulist[k][0][2] == False:
				self.selectedEntries.append(self.menulist[k][0][0])
				
		if config.plugins.configmanager.rememberselection.value:
			self.saveCurrentSelection()
			
		self.updateList(self.currentView, jumpToFirst=False)
				
	def deselectAll(self):
		self.selectedEntries = []
				
		if config.plugins.configmanager.rememberselection.value:
			self.saveCurrentSelection()
			
		self.updateList(self.currentView, jumpToFirst=False)
		
	def selectForBackUp(self):
		selectedMenuEntry = self["menulist"].getSelectedIndex()
		if self.menulist[selectedMenuEntry][0][2] == False:
			self.selectedEntries.append(self.menulist[selectedMenuEntry][0][0])
		else:
			for k in range(len(self.selectedEntries)):
				if self.selectedEntries[k] == self.menulist[selectedMenuEntry][0][0]:
					del self.selectedEntries[k]
					break
		#save it to config
		if config.plugins.configmanager.rememberselection.value:
			self.saveCurrentSelection()

		self.updateList(self.currentView, jumpToFirst=False)

	def saveCurrentSelection(self):
		if not fileExists("/usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection"):
			os.system("touch /usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection")
		
		os.system("echo %s > /usr/lib/enigma2/python/Plugins/Extensions/ConfigManager/selection" %(self.selectedEntries))
		
	def startBackUp(self):
		self.session.openWithCallback(self.setYellowButton, ConfigBackUpScreen, self.selectedEntries)
	
	def setYellowButton(self):
		self.backupdir = ""
		self.backupdir = config.plugins.configmanager.backuppath.value
		if fileExists("%s/settings.tar.gz" %(self.backupdir)):
			self["ButtonYellow"].show()
			self["ButtonYellowText"].show()
	
	def removeEntries(self):
		confirmbox = self.session.openWithCallback(self.removeEntriesConfirmed,MessageBox,_("%d entries will be removed. Proceed?") %(len(self.selectedEntries)), MessageBox.TYPE_YESNO)
		confirmbox.setTitle(_("Config Manager"))
		
	def removeEntriesConfirmed(self, answer):
		if answer is True:
			configfile.save()
			if fileExists("/etc/enigma2/settings"):
				try:
					buf = []
					origFile = open('/etc/enigma2/settings', 'r')
					self.settingsList = []
					self.settingsList = origFile.readlines()
				
					for i in range(len(self.selectedEntries)):
						for k in range(len(self.settingsList)):
							entrysplit = self.settingsList[k].split('=')
							if entrysplit[0] == self.selectedEntries[i]:
								#a dirty hack
								self.settingsList[k] = "#"
								break
				
					origFile.close()
				
					os.system("touch /etc/enigma2/settings.new")
					os.system("chmod 644 /etc/enigma2/setttings.new")
					configFile = open('/etc/enigma2/settings.new', 'w')	
					for j in range(len(self.settingsList)):
						if self.settingsList[j] != "#":
							configFile.write(self.settingsList[j])
					configFile.close()

					self.doRestart()			
				
				except IOError:
					print "couldn't read settings"
			
	def runCleanUpWizard(self):
		self.session.open(CleanUpWizardScreen, self.pluginConfigEntries)
		
	def showRestoreSelection(self):
		self.session.open(RestoreSelectionScreen)
		
	def closePlugin(self):
		self.close()
		
	def doRestart(self):
		restartbox = self.session.openWithCallback(self.restartBox,MessageBox,_("A reboot is required to load settings\nDo you want to reboot the Dreambox now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Reboot Dreambox now?"))
		
	def restartBox(self,answer):
		if answer is True:
			self.session.open(TryQuitMainloop, 2)
		
def Plugins(**kwargs):
    return [PluginDescriptor(name="Config Manager", description=_("Plugin to view/edit settings"), where = [PluginDescriptor.WHERE_PLUGINMENU], fnc=main)]
